/* Function file read_amber8_mden.c begun on 20070911 by BLFoley

This function reads information contained in fileset F.  The file
pointer in the fileset must already be pointing to a file open with
at least read capability.  The file should be a mden (detailed 
energy) output file generated by sander.  This function was written
for the AMBER 8 mden file format and should work with any other 
version that uses the same format.

The other arguments are NENT, the number of entries to read, and
a pointer to a set of NENT scharsets (ENT).  These charsets contain
the list of keywords for the type of data to be stored in the
return array.  NOTE!!!  The entries in ENT can be in any order,
but in the return array the information will be IN THE ORDER IN WHICH 
IT APPEARS IN THE MDEN FILE.  The entries in ENT must exactly match
the header keywords in an mden file.  

The return array will be two-dimensional, of size NENT*NMAX.  The
entries will be ordered such that the array can be accessed by
ARYP[column][row].  

If NENT is set to zero (and ENT is an empty char**), then the program
will read the contents of the entire mden file.

Example:

You want to read in Nsteps, Temp, pres_Z and E_hbon from the mden file.
The mden file contains 430 entries.

Call the function:

	double *myarray;
	char **ent;
	fileset MyMDEN;

	ent=(char*)calloc(4,sizeof(char*));	
	ent[0]=strdup("Nsteps");
	ent[1]=strdup("Temp");
	ent[2]=strdup("pres_Z");
	ent[3]=strdup("E_hbon");
	MyMDEN.N=strdup("My_sander_eout.en");
	MyMDEN.F=myfopen(MyMDEN.N,"r");
	
	myarray=read_amber8_mden(4,ent,MyMDEN);

The contents of myarray will look like:

column contains -->  	[index]	Nsteps	Temp	pres_Z	E_hbon
  mden entry
      |			[0]	1	320	0.98	42.6
      V			[1]	300	365	1.54	54.3
			[...]	...	...	...	...
			[429]	129000	319	1.25	48.9

The mden file looks like:

L0  Nsteps           time(ps)         Etot             EKinetic
L1  Temp             T_solute         T_solv           Pres_scal_solu
L2  Pres_scal_solv   BoxX             BoxY             BoxZ
L3  volume           pres_X           pres_Y           pres_Z
L4  Pressure         EKCoM_x          EKCoM_y          EKCoM_z
L5  EKComTot         VIRIAL_x         VIRIAL_y         VIRIAL_z
L6  VIRIAL_tot       E_pot            E_vdw            E_el
L7  E_hbon           E_bon            E_angle          E_dih
L8  E_14vdw          E_14el           E_const          E_pol
L9  AV_permMoment    AV_indMoment     AV_totMoment     Density          dV/dlambda
L0        1  0.2330001000E+04  -.6429480312E+05  0.1684971086E+05
L1 0.2977702692E+03  0.2977702692E+03  -.6434448936E+11  0.1000000000E+01
L2 0.1000000000E+01  0.6062081490E+02  0.7469235400E+02  0.6063588770E+02
L3 0.0000000000E+00  0.0000000000E+00  0.0000000000E+00  0.0000000000E+00
L4 0.0000000000E+00  0.0000000000E+00  0.0000000000E+00  0.0000000000E+00
L5 0.0000000000E+00  0.0000000000E+00  0.0000000000E+00  0.0000000000E+00
L6 0.0000000000E+00  -.8114451398E+05  0.9217360369E+04  -.1082986506E+06
L7 0.0000000000E+00  0.8145159202E+03  0.2010802568E+04  0.9283273104E+03
L8 0.1272671427E+04  0.1291045899E+05  0.0000000000E+00  0.0000000000E+00
L9 0.0000000000E+00  0.0000000000E+00  0.0000000000E+00  0.0000000000E+00  -.3245291868E+02
(etc)
*/
#include <AMBER/read_amber8_mden.h>
//#include "../inc/read_amber8_mden.h"
//#include "mylib.h"

double **read_amber8_mden(int NENT,char **ENT,int NDATA,fileset F){
int ra=0,rb=0,rc=0,NMAX=0,S[41];
double **ARYP,toss; // pointer to return array
char line[501];
char *MDEN_ENT[41];

//printf("Here, read mden TOP\n");

// initialize some stuff
MDEN_ENT[0]=strdup("Nsteps");
MDEN_ENT[1]=strdup("time(ps)");
MDEN_ENT[2]=strdup("Etot");
MDEN_ENT[3]=strdup("EKinetic"); 
MDEN_ENT[4]=strdup("Temp");
MDEN_ENT[5]=strdup("T_solute");
MDEN_ENT[6]=strdup("T_solv");
MDEN_ENT[7]=strdup("Pres_scal_solu"); 
MDEN_ENT[8]=strdup("Pres_scal_solv");
MDEN_ENT[9]=strdup("BoxX");

MDEN_ENT[10]=strdup("BoxY");
MDEN_ENT[11]=strdup("BoxZ"); 
MDEN_ENT[12]=strdup("volume");
MDEN_ENT[13]=strdup("pres_X");
MDEN_ENT[14]=strdup("pres_Y");
MDEN_ENT[15]=strdup("pres_Z"); 
MDEN_ENT[16]=strdup("Pressure");
MDEN_ENT[17]=strdup("EKCoM_x");
MDEN_ENT[18]=strdup("EKCoM_y");
MDEN_ENT[19]=strdup("EKCoM_z");

MDEN_ENT[20]=strdup("EKComTot");
MDEN_ENT[21]=strdup("VIRIAL_x");
MDEN_ENT[22]=strdup("VIRIAL_y");
MDEN_ENT[23]=strdup("VIRIAL_z"); 
MDEN_ENT[24]=strdup("VIRIAL_tot");
MDEN_ENT[25]=strdup("E_pot");
MDEN_ENT[26]=strdup("E_vdw");
MDEN_ENT[27]=strdup("E_el"); 
MDEN_ENT[28]=strdup("E_hbon");
MDEN_ENT[29]=strdup("E_bon");

MDEN_ENT[30]=strdup("E_angle");
MDEN_ENT[31]=strdup("E_dih"); 
MDEN_ENT[32]=strdup("E_14vdw");
MDEN_ENT[33]=strdup("E_14el");
MDEN_ENT[34]=strdup("E_const");
MDEN_ENT[35]=strdup("E_pol"); 
MDEN_ENT[36]=strdup("AV_permMoment");
MDEN_ENT[37]=strdup("AV_indMoment");
MDEN_ENT[38]=strdup("AV_totMoment");
MDEN_ENT[39]=strdup("Density");
MDEN_ENT[40]=strdup("dV/dlambda");

if(NENT==0){
	NENT=41;
	ENT=MDEN_ENT;
	}
ARYP=(double**)calloc(NENT,sizeof(double*));
//printf("Here, read mden 1\n");
// see which entries we need to grab
rb=0;
for(ra=0;ra<41;ra++){
	if(strcmp(MDEN_ENT[ra],ENT[rb])==0){
		S[ra]=0;
		rb++;}
	else{S[ra]=1;}
	}
if(rb!=NENT) mywhine("Mismatch in NENT & rb in mden read");
//printf("Here, read mden 2\n");

F.F=myfreopen(F.N,"r",F.F);
// look through the file to see how much memory to allocate
rewind(F.F);
//fgets(line,500,F.F); // get zeroth line
ra=0;
while(fgets(line,500,F.F)!=NULL){ if(strncmp(line,"L9",2)==0){ ra++; } }
NMAX=ra-1;
//printf("NMAX is %d\n",NMAX);
if(NMAX<NDATA){mywhine("read_amber8_mden mismatch in number of expected data points");} 
for(ra=0;ra<NENT;ra++){ ARYP[ra]=(double*)calloc(NMAX,sizeof(double)); }
rewind(F.F);

//// grab first 51 file entries and toss
//for(ra=0;ra<51;ra++){fscanf(F.F,"%s",line);}
// grab first 51 file entries and toss
for(ra=0;ra<10;ra++){fgets(line,500,F.F);}
// step through file, recording entries as needed
for(ra=0;ra<NMAX;ra++){
	rc=0;
	fscanf(F.F,"%s",line); // toss "L0"
	if(strcmp(line,"L0")!=0){
		printf("In read_amber8_mden, expect >>L0<< found >>%s<<\n",line);
		exit(1);
		}
	for(rb=0;rb<4;rb++){
		fscanf(F.F,"%lf",&toss);
		if(S[rb]==0){
			ARYP[rc][ra]=toss;
			rc++;
			}
		}
	fscanf(F.F,"%s",line); // toss "L1"
	if(strcmp(line,"L1")!=0){
		printf("In read_amber8_mden, expect >>L0<< found >>%s<<\n",line);
		exit(1);
		}
	for(rb=4;rb<8;rb++){
		fscanf(F.F,"%lf",&toss);
		if(S[rb]==0){
			ARYP[rc][ra]=toss;
			rc++;
			}
		}
	fscanf(F.F,"%s",line); // toss "L2"
	if(strcmp(line,"L2")!=0){
		printf("In read_amber8_mden, expect >>L0<< found >>%s<<\n",line);
		exit(1);
		}
	for(rb=8;rb<12;rb++){
		fscanf(F.F,"%lf",&toss);
		if(S[rb]==0){
			ARYP[rc][ra]=toss;
			rc++;
			}
		}
	fscanf(F.F,"%s",line); // toss "L3"
	if(strcmp(line,"L3")!=0){
		printf("In read_amber8_mden, expect >>L0<< found >>%s<<\n",line);
		exit(1);
		}
	for(rb=12;rb<16;rb++){
		fscanf(F.F,"%lf",&toss);
		if(S[rb]==0){
			ARYP[rc][ra]=toss;
			rc++;
			}
		}
	fscanf(F.F,"%s",line); // toss "L4"
	if(strcmp(line,"L4")!=0){
		printf("In read_amber8_mden, expect >>L0<< found >>%s<<\n",line);
		exit(1);
		}
	for(rb=16;rb<20;rb++){
		fscanf(F.F,"%lf",&toss);
		if(S[rb]==0){
			ARYP[rc][ra]=toss;
			rc++;
			}
		}
	fscanf(F.F,"%s",line); // toss "L5"
	if(strcmp(line,"L5")!=0){
		printf("In read_amber8_mden, expect >>L0<< found >>%s<<\n",line);
		exit(1);
		}
	for(rb=20;rb<24;rb++){
		fscanf(F.F,"%lf",&toss);
		if(S[rb]==0){
			ARYP[rc][ra]=toss;
			rc++;
			}
		}
	fscanf(F.F,"%s",line); // toss "L6"
	if(strcmp(line,"L6")!=0){
		printf("In read_amber8_mden, expect >>L0<< found >>%s<<\n",line);
		exit(1);
		}
	for(rb=24;rb<28;rb++){
		fscanf(F.F,"%lf",&toss);
		if(S[rb]==0){
			ARYP[rc][ra]=toss;
			rc++;
			}
		}
	fscanf(F.F,"%s",line); // toss "L7"
	if(strcmp(line,"L7")!=0){
		printf("In read_amber8_mden, expect >>L0<< found >>%s<<\n",line);
		exit(1);
		}
	for(rb=28;rb<32;rb++){
		fscanf(F.F,"%lf",&toss);
		if(S[rb]==0){
			ARYP[rc][ra]=toss;
			rc++;
			}
		}
	fscanf(F.F,"%s",line); // toss "L8"
	if(strcmp(line,"L8")!=0){
		printf("In read_amber8_mden, expect >>L0<< found >>%s<<\n",line);
		exit(1);
		}
	for(rb=32;rb<36;rb++){
		fscanf(F.F,"%lf",&toss);
		if(S[rb]==0){
			ARYP[rc][ra]=toss;
			rc++;
			}
		}
	fscanf(F.F,"%s",line); // toss "L9"
	if(strcmp(line,"L9")!=0){
		printf("In read_amber8_mden, expect >>L0<< found >>%s<<\n",line);
		exit(1);
		}
	for(rb=36;rb<41;rb++){
		fscanf(F.F,"%lf",&toss);
		if(S[rb]==0){
			ARYP[rc][ra]=toss;
			rc++;
			}
		}
	}

return ARYP;
}
